#include "TCanvas.h"
#include "TGraphErrors.h"
#include "TF1.h"

using namespace std;

void checkCalib(){

	//We construct a graph to store the calibration.
	//With this constructors we directly read the txt file.
    //Check the reference guide for the different options.
	TGraphErrors* gCal = new TGraphErrors("px_calibration.txt","%lg %lg %lg");
 
	//Comment the line above, and uncomment that below. Check the difference.
	//TGraphErrors* gCal = new TGraphErrors("px_calibration_errX.txt");
  
	//Some style choices
	gCal->GetXaxis()->SetTitle("Measured p_{x} [GeV/c]");
	gCal->GetYaxis()->SetTitle("Calibrated p_{x} [GeV/c]");
	gCal->GetYaxis()->SetRangeUser(0,6);
    gCal->SetTitle(0);
	gCal->SetMarkerStyle(8);
	gCal->SetMarkerSize(0.8);
    
	//We define a function to fit the calibration data
	TF1* f_calib = new TF1("f_calib","pol1",0,5);
    gCal->Fit("f_calib");
  
	//Let's draw the calibration graph and save it in a pdf.
	TCanvas* ccal = new TCanvas("ccal","ccal",800,600);
	gCal->Draw("AP");
    ccal->SaveAs("calibration.pdf");
  
	//it's very useful in a fit to draw the pulls:
	//the residual (fit - data) divided by the uncertainty
	//The pulls should be distributed as a normal gaussian.
	TGraph* gPulls = new TGraph(gCal->GetN());
	TH1D* hPulls = new TH1D("hPulls",";pull;counts", 10, -6, 6);
  
	for(int i=0; i<gCal->GetN(); ++i){
	
	  double ydata = gCal->GetPointY(i);
	  double xdata = gCal->GetPointX(i);
	  double error = gCal->GetErrorY(i);
	  
	  //if you have error on x, you need to consider it.
	  //Calculate it and add in quadrature to "error".

	  double pull = (ydata - f_calib->Eval(xdata))/error;
	  
	  gPulls->SetPoint(i, xdata, pull);
	  hPulls->Fill(pull);
	}
  
	TCanvas* cpull = new TCanvas("cpull","cpull",1200,500);
	cpull->Divide(2,1);
	cpull->cd(1);
	gPulls->GetXaxis()->SetTitle("Measured p_{x} [GeV/c]");
	gPulls->GetYaxis()->SetTitle("pull");
	gPulls->SetTitle(0);
	gPulls->SetMarkerStyle(8);
	gPulls->SetMarkerSize(0.8);
	gPulls->Draw("AP");
	
	cpull->cd(2);
	hPulls->Fit("gaus");
	hPulls->Draw();
  
    return;
}
